@@ -44,7 +44,7 @@ char * convertToUtf8(const char * input,
 
   cd = iconv_open("UTF-8", charset);
   if (cd == (iconv_t) -1)
-    return strdup(charset);
+    return strdup(input);
   tmpSize = 3 * len + 4;
   tmp = malloc(tmpSize);
   itmp = tmp;
@@ -56,7 +56,7 @@ char * convertToUtf8(const char * input,
 	    &finSize) == (size_t)-1) {
     iconv_close(cd);
     free(tmp);
-    return strdup(charset);
+    return strdup(input);
   }
   ret = malloc(tmpSize - finSize + 1);
   memcpy(ret,
