@@ -1,6 +1,6 @@
 /*
      This file is part of doodle.
-     (C) 2004, 2006 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2006, 2010 Christian Grothoff (and other contributing authors)
 
      doodle is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -57,9 +57,6 @@ static int very_verbose = 0;
  */
 static void printHelp () {
   static Help help[] = {
-    { 'B', "binary", "LANG",
-      gettext_noop("use the generic plaintext extractor"
-		   " for the language with the 2-letter language code LANG") },
     { 'd', "database", "FILENAME",
       gettext_noop("use location FILENAME to store doodle database") },
     { 'D', "debug", NULL,
@@ -68,8 +65,6 @@ static void printHelp () {
       gettext_noop("add the filename to the list of keywords") },
     { 'h', "help", NULL,
       gettext_noop("print this help page") },
-    { 'H', "hash", "ALGORITHM",
-      gettext_noop("compute hash using the given ALGORITHM") },
     { 'l', "library", "LIBRARY",
       gettext_noop("load an extractor plugin named LIBRARY") },
     { 'L', "log", "FILENAME",
@@ -758,7 +753,7 @@ static void detachFromTerminal(int * filedes) {
   }
   close(filedes[0]); /* we only write */
   nullfd = open("/dev/null",
-		O_CREAT | O_RDWR | O_APPEND);
+		O_CREAT | O_RDWR | O_APPEND, S_IWUSR | S_IRUSR);
   if (nullfd < 0) {
     perror("/dev/null");
     exit(1);
@@ -794,8 +789,6 @@ int main(int argc,
   char * libraries = NULL;
   size_t mem_limit = 0;
   char * dbName;
-  char * binary = NULL;
-  char * hash = NULL;
   char * tmp;
   char * log = NULL;
   int ret;
@@ -814,12 +807,10 @@ int main(int argc,
 
   while (1) {
     static struct option long_options[] = {
-      {"binary", 1, 0, 'B'},
       {"database", 1, 0, 'd'},
       {"debug", 0, 0, 'D'},
       {"filenames", 0, 0, 'f'} ,
       {"help", 0, 0, 'h'},
-      {"hash", 1, 0, 'H'},
       {"library", 1, 0, 'l'},
       {"log", 1, 0, 'L'},
       {"memory", 1, 0, 'm'},
@@ -831,16 +822,13 @@ int main(int argc,
     };
     option_index = 0;
     c = getopt_long(argc,
-		    argv, "B:d:DfhH:l:L:m:nP:vV",
+		    argv, "d:Dfhl:L:m:nP:vV",
 		    long_options,
 		    &option_index);
 
     if (c == -1)
       break; /* No more flags to process */
     switch (c) {
-    case 'B':
-      binary = optarg;
-      break;
     case 'd':
       dbName = optarg;
       break;
@@ -853,9 +841,6 @@ int main(int argc,
     case 'h':
       printHelp();
       return 0;
-    case 'H':
-      hash = optarg;
-      break;
     case 'l':
       libraries = optarg;
       break;
@@ -904,45 +889,6 @@ int main(int argc,
 
   if (libraries != NULL)
     libraries = strdup(libraries);
-  if (binary != NULL) {
-    if (libraries == NULL)
-      tmp = MALLOC(strlen("libextractor_printable_") + 1 +
-		   strlen(binary));
-    else
-      tmp = MALLOC(strlen(libraries) +
-		   strlen(":libextractor_printable_") + 1 +
-		   strlen(binary));
-    if (libraries == NULL) {
-      strcpy(tmp, "libextractor_printable_");
-    } else {
-      strcpy(tmp, libraries);
-      strcat(tmp, ":libextractor_printable_");
-    }
-    strcat(tmp, binary);
-    if (libraries != NULL)
-      free(libraries);
-    libraries = tmp;
-  }
-
-  if (hash != NULL) {
-    if (libraries == NULL)
-      tmp = MALLOC(strlen("libextractor_hash_") + 1 +
-		   strlen(hash));
-    else
-      tmp = MALLOC(strlen(libraries) +
-		   strlen(":libextractor_hash_") + 1 +
-		   strlen(hash));
-    if (libraries == NULL) {
-      strcpy(tmp, "libextractor_hash_");
-    } else {
-      strcpy(tmp, libraries);
-      strcat(tmp, ":libextractor_hash_");
-    }
-    strcat(tmp, hash);
-    if (libraries != NULL)
-      free(libraries);
-    libraries = tmp;
-  }
 
   if (do_debug == 0)
     detachFromTerminal(filedes);
