@@ -1,6 +1,6 @@
 /*
      This file is part of doodle.
-     (C) 2004 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2005 Christian Grothoff (and other contributing authors)
 
      doodle is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -69,6 +69,15 @@ typedef int (*ScannerCallback)(const char * filename,
  */
 char * expandFileName(const char * fil);
 
+/**
+ * Function called by scanDirectory to check if a given
+ * directory (and all subdirectories) is "pruned" and
+ * thus should not be considered in the scan.
+ * @return 0 if not pruned, 1 if pruned
+ */
+typedef int (*PruneCheck)(const char * name,
+			  void * cls);
+
 /**
  * Scan a directory for files. The name of the directory
  * must be expanded first (!).
@@ -81,16 +90,29 @@ char * expandFileName(const char * fil);
 int scanDirectory(char * dirName,
 		  DOODLE_Logger logger,
 		  void * context,
+		  PruneCheck pruner,
+		  void * pr_arg,
 		  ScannerCallback callback,
 		  void * cb_arg);
 
 
+struct EXTRACT_Process;
+
+struct EXTRACT_Process * forkExtractor(int do_default,
+				       const char * libraries,
+				       DOODLE_Logger logger,
+				       void * log_ctx);
+
+void joinExtractor(struct EXTRACT_Process * proc);
+
+
 /**
  * Find keywords in the given file and append
  * the string describing these keywords to the
  * long string (for the suffix tree).
  */
-int buildIndex(EXTRACTOR_ExtractorList * elist,
+int buildIndex(struct EXTRACT_Process * elist,
+	       FILE * logFile,
 	       const char * filename,
 	       struct DOODLE_SuffixTree * tree,
 	       int do_filenames);
