@@ -1,6 +1,6 @@
 /*
      This file is part of doodle.
-     (C) 2004, 2006 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2006, 2010 Christian Grothoff (and other contributing authors)
 
      doodle is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -36,6 +36,7 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include <signal.h>
 
 #define DEBUG_IPC 0
 
@@ -116,6 +117,42 @@ static int do_read(int fd,
 #define DO_READ(fd,data,len) if (do_read(fd,data,len)) goto ERROR;
 #define MAX_SLEN 16 * 1024 * 1024
 
+struct AccuCtx
+{
+  unsigned int count;
+  unsigned int size;
+  enum EXTRACTOR_MetaType *types;
+  char ** keywords;
+};
+
+
+static int 
+accumulator(void *cls,
+	    const char *plugin_name,
+	    enum EXTRACTOR_MetaType type,
+	    enum EXTRACTOR_MetaFormat format,
+	    const char *data_mime_type,
+	    const char *data,
+	    size_t data_len)
+{
+  struct AccuCtx * ac = cls;
+  unsigned int nsize;
+  if ( (format != EXTRACTOR_METAFORMAT_UTF8) &&
+       (format != EXTRACTOR_METAFORMAT_C_STRING) )
+    return 0;
+  if (ac->size == ac->count)
+    {
+      nsize = 4 + 2 * ac->size;
+      ac->types = realloc (ac->types, nsize * sizeof(enum EXTRACTOR_MetaType));
+      ac->keywords = realloc (ac->keywords, nsize * sizeof(char *));
+      ac->size = nsize;
+    }
+  ac->types[ac->count] = type;
+  ac->keywords[ac->count++] = strdup (data);
+  return 0;
+}
+
+
 /**
  * @return 0 on success, 1 on error.
  */
@@ -126,11 +163,9 @@ static int do_fork(struct EXTRACT_Process * proc) {
   size_t pos;
   ssize_t ret;
   size_t slen;
-  EXTRACTOR_ExtractorList * list;
-  EXTRACTOR_KeywordList * head;
-  EXTRACTOR_KeywordList * next;
+  struct EXTRACTOR_PluginList * list;
   char * filename;
-  unsigned int count;
+  struct AccuCtx acc_ctx;
 
 #if DEBUG_IPC
   fprintf(stderr, "Forking!\n");
@@ -191,12 +226,12 @@ static int do_fork(struct EXTRACT_Process * proc) {
   list = NULL;
 
   if (proc->do_default)
-    list = EXTRACTOR_loadDefaultLibraries();
+    list = EXTRACTOR_plugin_add_defaults(EXTRACTOR_OPTION_DEFAULT_POLICY);
   if (proc->libs != NULL) 
-    list = EXTRACTOR_loadConfigLibraries(list,
-					 proc->libs);  
+    list = EXTRACTOR_plugin_add_config(list,
+				       proc->libs,
+				       EXTRACTOR_OPTION_DEFAULT_POLICY);  
   
-  head = NULL;
   pos = 0;
   buffer[FILENAME_MAX + 1] = '\0';
   ret = read(filedes2[0], &buffer[pos], FILENAME_MAX + 1 - pos);
@@ -210,33 +245,40 @@ static int do_fork(struct EXTRACT_Process * proc) {
 	      &buffer[slen],
 	      pos - slen);
       pos = pos - slen;
-      head = EXTRACTOR_getKeywords(list,
-				   filename);
-      head = EXTRACTOR_removeDuplicateKeywords
-	(head,
-	 EXTRACTOR_DUPLICATES_REMOVE_UNKNOWN);
-      count = EXTRACTOR_countKeywords(head);
-      DO_WRITE(filedes1[1], &count, sizeof(unsigned int));
-      while (head != NULL) {
-	next = head->next;
-	DO_WRITE(filedes1[1], &head->keywordType, sizeof(EXTRACTOR_KeywordType));
-	slen = strlen(head->keyword);
+      acc_ctx.count = 0;
+      acc_ctx.size = 0;
+      acc_ctx.keywords = NULL;
+      acc_ctx.types = NULL;
+      EXTRACTOR_extract(list,
+			filename,
+			NULL, 0,
+			&accumulator,
+			&acc_ctx);
+      DO_WRITE(filedes1[1], &acc_ctx.count, sizeof(unsigned int));
+      while (0 != acc_ctx.count--) {
+	DO_WRITE(filedes1[1], 
+		 &acc_ctx.types[acc_ctx.count],
+		 sizeof(enum EXTRACTOR_MetaType));
+	slen = strlen(acc_ctx.keywords[acc_ctx.count]);
 	if (slen > MAX_SLEN)
 	  slen = MAX_SLEN; /* cut off -- far too large! */
 	DO_WRITE(filedes1[1], &slen, sizeof(size_t));
-	DO_WRITE(filedes1[1], head->keyword, slen);
-	free(head->keyword);
-	free(head);
-	head = next;
+	DO_WRITE(filedes1[1], acc_ctx.keywords[acc_ctx.count], slen);
+	free(acc_ctx.keywords[acc_ctx.count]);
       }
+      free (acc_ctx.keywords);
+      free (acc_ctx.types);
+      acc_ctx.keywords = NULL;
+      acc_ctx.types = NULL;
+      acc_ctx.size = 0;
     }
     ret = read(filedes2[0], &buffer[pos], FILENAME_MAX + 1 - pos);
   }
   /* exit / cleanup */
  ERROR:
-  if (head != NULL)
-    EXTRACTOR_freeKeywords(head);
-  EXTRACTOR_removeAll(list);
+  free (acc_ctx.keywords);
+  free (acc_ctx.types);
+  EXTRACTOR_plugin_remove_all (list);
   close(filedes2[0]);
   close(filedes1[1]);
   /* never return - we were forked! */
@@ -244,12 +286,38 @@ static int do_fork(struct EXTRACT_Process * proc) {
   return 1; /* eh, dead */
 }
 
-static EXTRACTOR_KeywordList * 
+
+struct KeywordList {
+  /* the keyword that was found */
+  char * keyword;
+  /* the type of the keyword (classification) */
+  enum EXTRACTOR_MetaType keywordType;
+  /* the next entry in the list */
+  struct KeywordList * next;
+};
+
+
+static void 
+freeKeywords (struct KeywordList * keywords)
+{
+  struct KeywordList *prev;
+  while (keywords != NULL)
+    {
+      prev = keywords;
+      keywords = keywords->next;
+      free (prev->keyword);
+      free (prev);
+    }
+}
+
+
+
+static struct KeywordList * 
 getKeywords(struct EXTRACT_Process * eproc,
 	    const char * filename) {
-  EXTRACTOR_KeywordList * pos;
-  EXTRACTOR_KeywordList * next;
-  EXTRACTOR_KeywordList * head;
+  struct KeywordList * pos;
+  struct KeywordList * next;
+  struct KeywordList * head;
   unsigned int count;
   size_t slen;
   int status;
@@ -260,14 +328,14 @@ getKeywords(struct EXTRACT_Process * eproc,
   if (eproc->pid == -1)
     return NULL;
   fprintf(stderr, "Processing file %s\n", filename);
-  DO_WRITE(eproc->send_pipe, filename, strlen(filename)+1);
   head = NULL;
   pos = NULL;
   next = NULL;
+  DO_WRITE(eproc->send_pipe, filename, strlen(filename)+1);
   DO_READ(eproc->read_pipe, &count, sizeof(unsigned int));
   while (count-- > 0) {
-    next = malloc(sizeof(EXTRACTOR_KeywordList));
-    DO_READ(eproc->read_pipe, &next->keywordType, sizeof(EXTRACTOR_KeywordType));
+    next = malloc(sizeof(struct KeywordList));
+    DO_READ(eproc->read_pipe, &next->keywordType, sizeof(enum EXTRACTOR_MetaType));
     DO_READ(eproc->read_pipe, &slen, sizeof(size_t));
     if (slen == 0)
     {
@@ -298,7 +366,7 @@ getKeywords(struct EXTRACT_Process * eproc,
 #endif
   if (next != NULL)
     free(next);
-  EXTRACTOR_freeKeywords(head);
+  freeKeywords(head);
   if (eproc->send_pipe != -1)
     close(eproc->send_pipe);
   eproc->send_pipe = -1;
@@ -324,8 +392,8 @@ int buildIndex(struct EXTRACT_Process * eproc,
 	       const char * filename,
 	       struct DOODLE_SuffixTree * tree,
 	       int do_filenames) {
-  EXTRACTOR_KeywordList * head;
-  EXTRACTOR_KeywordList * pos;
+  struct KeywordList * head;
+  struct KeywordList * pos;
 
   head = getKeywords(eproc, filename); 
   pos  = head;
@@ -333,10 +401,6 @@ int buildIndex(struct EXTRACT_Process * eproc,
     char * cpos;
     size_t slen;
 
-    if (EXTRACTOR_isBinaryType(pos->keywordType)) {
-      pos = pos->next;
-      continue;
-    }
     cpos = pos->keyword;
     if (logFile != NULL)
       fprintf(logFile, "%s\n", cpos);
@@ -356,7 +420,7 @@ int buildIndex(struct EXTRACT_Process * eproc,
 	  if (0 != DOODLE_tree_expand(tree,
 				      xpos,
 				      filename)) {
-	    EXTRACTOR_freeKeywords(head);
+	    freeKeywords(head);
 	    return 0;
 	  }
 	  xpos++;
@@ -367,7 +431,7 @@ int buildIndex(struct EXTRACT_Process * eproc,
 	if (0 != DOODLE_tree_expand(tree,
 				    cpos,
 				    filename)) {
-	  EXTRACTOR_freeKeywords(head);
+	  freeKeywords(head);
 	  return 0;
 	}
 	cpos++;
@@ -375,7 +439,7 @@ int buildIndex(struct EXTRACT_Process * eproc,
     }
     pos = pos->next;
   }
-  EXTRACTOR_freeKeywords(head);
+  freeKeywords(head);
   if (do_filenames) {
     const char * cpos = filename;
     while (cpos[0] != '\0') {
