@@ -1,6 +1,6 @@
 /*
      This file is part of doodle.
-     (C) 2004 Christian Grothoff (and other contributing authors)
+     (C) 2004, 2007 Christian Grothoff (and other contributing authors)
 
      doodle is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -38,7 +38,6 @@
 
 #include "config.h"
 #include "doodle.h"
-#include <netinet/in.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -447,6 +446,18 @@ static int READULONG(BIO * fd,
   return 0;
 }
 
+static int READULONGFULL(BIO * fd,
+                         unsigned long long * val) {
+  unsigned int d, e;
+  unsigned char v[8];
+  *val = 0;
+  if (-1 == READALL(fd, &v[0], 8))
+    return -1;
+  for (d=0,e=7;d<8;d++,e--)
+    (*val) += (((unsigned long long)v[d]) << (8*e));
+  return 0;
+}
+
 static int READUINTPAIR(BIO * fd,
 			unsigned int * val1,
 			unsigned int * val2) {
@@ -601,6 +612,16 @@ static void WRITEULONG(BIO * fd,
   WRITEALL(fd, &v[0], (int) c);
 }
 
+static void WRITEULONGFULL(BIO * fd,
+                           unsigned long long val) {
+  unsigned int d, e;
+  unsigned char v[8];
+
+  for (d=0,e=7;d<8;d++,e--)
+    v[d] = (unsigned char) (val >> (8*e));
+  WRITEALL(fd, &v[0], 8);
+}
+
 static char * readZT(BIO * fd) {
   unsigned int len;
   char * buf;
@@ -1517,7 +1538,7 @@ static unsigned long long writeNode(BIO * fd,
  * time introducing the 'mls' node groups (which has the potential
  * to significantly improve performance).
  */
-static char * MAGIC = "DOO\0000006";
+static char * MAGIC = "DOO\0000007";
 
 /**
  * Magic string to indicate an temporary doodle database that
@@ -1740,9 +1761,7 @@ SuffixTree * DOODLE_tree_create_internal(DOODLE_Logger log,
   	return NULL;
       }
     }
-    if (-1 == READALL(fd,
-		      &off,
-		      sizeof(unsigned int))) {
+    if (-1 == READULONGFULL(fd, &off)) {
       for (i=ret->cisPos-1;i>=0;i--)
 	free(ret->cis[++i]);
       free(ret->cis);
@@ -1754,7 +1773,6 @@ SuffixTree * DOODLE_tree_create_internal(DOODLE_Logger log,
       IO_FREE(fd);
       return NULL;
     }
-    off = ntohl(off);
     ret->fd = fd;
     ret->root = lazyReadNode(ret,
 			     off);
@@ -1983,18 +2001,13 @@ void DOODLE_tree_destroy(SuffixTree * tree) {
 	      tree->cis[i]);
     wpos = LSEEK(fd, 0, SEEK_CUR);
     off = 0;
-    WRITEALL(fd,
-	     &off,
-	     sizeof(unsigned long long));
+    WRITEULONGFULL(fd, off);
 
     off = writeNode(fd,
 		    tree,
 		    tree->root);
     LSEEK(fd, wpos, SEEK_SET);
-    off = htonl(off);
-    WRITEALL(fd,
-	     &off,
-	     sizeof(unsigned long long));
+    WRITEULONGFULL(fd, off);
     IO_FREE(tree->fd);
     tree->fd = NULL;
     IO_FREE(fd);
